"""Views supporting template previews in admin site."""
from __future__ import annotations

import json
import logging

from django.conf import settings as django_settings
from django.contrib import messages
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.http.response import HttpResponseBadRequest
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.decorators.clickjacking import xframe_options_sameorigin

from .forms import EmailTestForm, MultiEmailTemplateField
from .helpers import merge_dicts
from .models import EmailTemplate, LoggedMessage

logger = logging.getLogger(__name__)


@user_passes_test(lambda u: u.is_staff)
@xframe_options_sameorigin
def render_template_subject(request: HttpRequest, template_id: int) -> HttpResponse:
    """Render the template subject."""
    template = get_object_or_404(EmailTemplate, id=template_id)
    html = template.render_subject(template.test_context)
    return HttpResponse(html, content_type="text/plain")


@user_passes_test(lambda u: u.is_staff)
@xframe_options_sameorigin
def render_template_body(
    request: HttpRequest, template_id: int, content_type: str
) -> HttpResponse:
    """Render the template body as plain text or HTML."""
    template = get_object_or_404(EmailTemplate, id=template_id)
    if content_type in (
        EmailTemplate.CONTENT_TYPE_PLAIN,
        EmailTemplate.CONTENT_TYPE_HTML,
    ):
        html = template.render_body(template.test_context, content_type)
        return HttpResponse(html, content_type=content_type)
    # do not return the content_type to the user, as it is
    # user-generated and _could_ be a vulnerability.
    return HttpResponse("Invalid content_type specified.", status=400)


@user_passes_test(lambda u: u.is_staff)
def send_test_email(request: HttpRequest) -> HttpResponseRedirect:
    """Intermediate admin action page for sending a single test email."""
    # use the field.to_python here as belt-and-braces - if it works here
    # we can be confident that it'll work on the POST.
    templates = MultiEmailTemplateField().to_python(request.GET["templates"])

    if request.method == "GET":
        contexts = merge_dicts(*[t.test_context for t in templates])
        context = json.dumps(contexts, indent=4, sort_keys=True)
        initial = {"templates": request.GET["templates"], "context": context}
        try:
            template = templates.get()
            initial["from_email"] = template.from_email
            initial["reply_to"] = template.reply_to
        except EmailTemplate.MultipleObjectsReturned:
            initial["from_email"] = django_settings.DEFAULT_FROM_EMAIL
            initial["reply_to"] = django_settings.DEFAULT_FROM_EMAIL
        form = EmailTestForm(initial=initial)
        return render(
            request,
            "appmail/send_test_email.html",
            {
                "form": form,
                "templates": templates,
                # opts are used for rendering some page furniture - breadcrumbs etc.
                "opts": EmailTemplate._meta,
            },
        )

    if request.method == "POST":
        form = EmailTestForm(request.POST)
        if form.is_valid():
            form.send_emails(request)
            return HttpResponseRedirect(
                "{}?{}".format(
                    reverse("appmail:send_test_email"), request.GET.urlencode()
                )
            )
        else:
            return render(
                request,
                "appmail/send_test_email.html",
                {
                    "form": form,
                    "templates": templates,
                    # opts are used for rendering some page furniture - breadcrumbs etc.
                    "opts": EmailTemplate._meta,
                },
                status=422,
            )


@user_passes_test(lambda u: u.is_staff)
def resend_email(request: HttpRequest, email_id: int) -> HttpResponseRedirect:
    """Resend a specific LoggedMessage."""
    email = LoggedMessage.objects.get(id=email_id)
    email.resend()
    messages.success(request, _("Resent email to {}".format(email.to)))
    return HttpResponseRedirect(reverse("admin:appmail_loggedmessage_changelist"))


@user_passes_test(lambda u: u.is_staff)
@xframe_options_sameorigin
def render_message_body(
    request: HttpRequest, email_id: int, content_type: str
) -> HttpResponse:
    """Render the email body as plain text or HTML."""
    email = get_object_or_404(LoggedMessage, id=email_id)
    if content_type == EmailTemplate.CONTENT_TYPE_PLAIN:
        return HttpResponse(email.body, content_type=content_type)
    if content_type == EmailTemplate.CONTENT_TYPE_HTML:
        return HttpResponse(email.html, content_type=content_type)
    return HttpResponseBadRequest("Invalid content_type specified.")



from django.urls import path

import appmail.models
import appmail.views

app_name = "appmail"

urlpatterns = [
    # template paths
    path(
        "templates/<int:template_id>/body.txt",
        appmail.views.render_template_body,
        kwargs={"content_type": "text/plain"},
        name="render_template_body_text",
    ),
    path(
        "templates/<int:template_id>/body.html",
        appmail.views.render_template_body,
        kwargs={"content_type": "text/html"},
        name="render_template_body_html",
    ),
    path(
        "templates/<int:template_id>/subject.txt",
        appmail.views.render_template_subject,
        name="render_template_subject",
    ),
    path("templates/test/", appmail.views.send_test_email, name="send_test_email"),
    # LoggedMessage paths
    path(
        "emails/<int:email_id>/body.txt",
        appmail.views.render_message_body,
        kwargs={"content_type": appmail.models.EmailTemplate.CONTENT_TYPE_PLAIN},
        name="render_message_body_text",
    ),
    path(
        "emails/<int:email_id>/body.html",
        appmail.views.render_message_body,
        kwargs={"content_type": appmail.models.EmailTemplate.CONTENT_TYPE_HTML},
        name="render_message_body_html",
    ),
    path(
        "emails/resend/<int:email_id>/", appmail.views.resend_email, name="resend_email"
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-08-07 06:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("appmail", "0003_emailtemplate_test_context")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="is_active",
            field=models.BooleanField(
                default=True,
                help_text="Set to False to remove from `current` queryset.",
                verbose_name="Active (live)",
            ),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-06 08:15
from __future__ import unicode_literals

from django.db import migrations

from ..compat import JSONField


class Migration(migrations.Migration):

    dependencies = [("appmail", "0002_add_template_description")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="test_context",
            field=JSONField(
                default=dict,
                blank=True,
                help_text=(
                    "Dummy JSON used for test rendering (set automatically on first "
                    "save)."
                ),
            ),
        )
    ]



from __future__ import annotations

import json
from typing import Optional, Tuple

from django.contrib import admin, messages
from django.core.exceptions import ValidationError
from django.db.models.query import QuerySet
from django.http import HttpRequest, HttpResponseRedirect
from django.template.defaultfilters import truncatechars
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _lazy

from .compat import JSONField
from .forms import JSONWidget
from .models import EmailTemplate, LoggedMessage


class ValidTemplateListFilter(admin.SimpleListFilter):
    """Filter on whether the template can be rendered or not."""

    title = _lazy("Is valid")
    parameter_name = "valid"

    def lookups(
        self, request: HttpRequest, model_admin: admin.ModelAdmin
    ) -> Tuple[Tuple[str, str], Tuple[str, str]]:
        """
        Return valid template True/False filter values tuples.

        The first element in each tuple is the coded value for the option
        that will appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.

        """
        return (("1", _lazy("True")), ("0", _lazy("False")))

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:
        """
        Return the filtered queryset.

        Filter based on the value provided in the query string and
        retrievable via `self.value()`.

        """
        valid_ids = []
        invalid_ids = []
        for obj in queryset:
            try:
                obj.clean()
                valid_ids.append(obj.pk)
            except ValidationError:
                invalid_ids.append(obj.pk)

        if self.value() == "1":
            return queryset.filter(pk__in=valid_ids)
        if self.value() == "0":
            return queryset.filter(pk__in=invalid_ids)


class AdminBase(admin.ModelAdmin):
    def iframe(self, url: str) -> str:
        """Return an iframe containing the url for display in change view."""
        return format_html(
            f"<iframe class='appmail' src='{url}' onload='resizeIframe(this)'></iframe>"
            f"<br/><a href='{url}' target='_blank'>View in new tab.</a>"
        )

    def pretty_print(self, data: Optional[dict]) -> str:
        """Convert dict into formatted HTML."""
        if data is None:
            return "(None)"
        pretty = json.dumps(data, sort_keys=True, indent=4, separators=(",", ": "))
        html = pretty.replace(" ", "&nbsp;").replace("\n", "<br>")
        return mark_safe("<pre><code>%s</code></pre>" % html)


@admin.register(EmailTemplate)
class EmailTemplateAdmin(AdminBase):

    formfield_overrides = {JSONField: {"widget": JSONWidget}}

    list_display = (
        "name",
        "subject",
        "language",
        "version",
        "has_text",
        "has_html",
        "is_valid",
        "is_active",
    )

    list_filter = ("language", "version", ValidTemplateListFilter, "is_active")

    readonly_fields = ("render_subject", "render_text", "render_html")

    search_fields = ("name", "subject")

    actions = (
        "activate_templates",
        "deactivate_templates",
        "clone_templates",
        "send_test_emails",
    )

    fieldsets = (
        (
            "Basic Information",
            {"fields": ("name", "description", "language", "version", "is_active")},
        ),
        ("Email Defaults", {"fields": ("from_email", "reply_to")}),
        ("Templates", {"fields": ("subject", "body_text", "body_html")}),
        (
            "Sample Output",
            {
                "fields": (
                    "test_context",
                    "render_subject",
                    "render_text",
                    "render_html",
                )
            },
        ),
    )

    # these functions are here rather than on the model so that we can get the
    # boolean icon.
    def has_text(self, obj: EmailTemplate) -> bool:
        return len(obj.body_text or "") > 0

    has_text.boolean = True  # type: ignore

    def has_html(self, obj: EmailTemplate) -> bool:
        return len(obj.body_html or "") > 0

    has_html.boolean = True  # type: ignore

    def is_valid(self, obj: EmailTemplate) -> bool:
        """Return True if the template can be rendered."""
        try:
            obj.clean()
            return True
        except ValidationError:
            return False

    is_valid.boolean = True  # type: ignore

    def render_subject(self, obj: EmailTemplate) -> str:
        if obj.id is None:
            url = ""
        else:
            url = reverse(
                "appmail:render_template_subject", kwargs={"template_id": obj.id}
            )
        return self.iframe(url)

    render_subject.short_description = "Rendered subject"  # type: ignore
    render_subject.allow_tags = True  # type: ignore

    def render_text(self, obj: EmailTemplate) -> str:
        if obj.id is None:
            url = ""
        else:
            url = reverse(
                "appmail:render_template_body_text", kwargs={"template_id": obj.id}
            )
        return self.iframe(url)

    render_text.short_description = "Rendered body (plain)"  # type: ignore
    render_text.allow_tags = True  # type: ignore

    def render_html(self, obj: EmailTemplate) -> str:
        if obj.id is None:
            url = ""
        else:
            url = reverse(
                "appmail:render_template_body_html", kwargs={"template_id": obj.id}
            )
        return self.iframe(url)

    render_html.short_description = "Rendered body (html)"  # type: ignore
    render_html.allow_tags = True  # type: ignore

    def send_test_emails(
        self, request: HttpRequest, queryset: QuerySet
    ) -> HttpResponseRedirect:
        selected = ",".join([str(s) for s in queryset.values_list("id", flat=True)])
        url = "{}?templates={}".format(reverse("appmail:send_test_email"), selected)
        return HttpResponseRedirect(url)

    send_test_emails.short_description = _lazy(  # type: ignore
        "Send test email for selected templates"
    )

    def clone_templates(
        self, request: HttpRequest, queryset: QuerySet
    ) -> HttpResponseRedirect:
        for template in queryset:
            template.clone()
            messages.success(request, _lazy("Cloned template '%s'" % template.name))
        return HttpResponseRedirect(request.path)

    clone_templates.short_description = _lazy(  # type: ignore
        "Clone selected email templates"
    )

    def activate_templates(
        self, request: HttpRequest, queryset: QuerySet
    ) -> HttpResponseRedirect:
        count = queryset.update(is_active=True)
        messages.success(request, _lazy("Activated %s templates" % count))
        return HttpResponseRedirect(request.path)

    activate_templates.short_description = _lazy(  # type: ignore
        "Activate selected email templates"
    )

    def deactivate_templates(
        self, request: HttpRequest, queryset: QuerySet
    ) -> HttpResponseRedirect:
        count = queryset.update(is_active=False)
        messages.success(request, _lazy("Deactivated %s templates" % count))
        return HttpResponseRedirect(request.path)

    deactivate_templates.short_description = _lazy(  # type: ignore
        "Deactivate selected email templates"
    )


@admin.register(LoggedMessage)
class LoggedMessageAdmin(AdminBase):

    exclude = ("html", "context")

    formfield_overrides = {JSONField: {"widget": JSONWidget}}

    list_display = ("to", "template_name", "_subject", "timestamp")

    list_filter = ("timestamp", "template__name", "template__language")

    raw_id_fields = ("user", "template")

    readonly_fields = (
        "to",
        "user",
        "template",
        "template_context",
        "subject",
        "body",
        "render_html",
        "timestamp",
    )

    search_fields = ("to", "subject")

    def _subject(self, obj: LoggedMessage) -> str:
        """Truncate the subject for display."""
        return truncatechars(obj.subject, 50)

    def template_name(self, obj: LoggedMessage) -> str:
        return obj.template.name

    def template_context(self, obj: LoggedMessage) -> str:
        """Pretty print version of the template context dict."""
        return self.pretty_print(obj.context)

    def render_html(self, obj: LoggedMessage) -> str:
        if obj.id is None:
            url = ""
        else:
            url = reverse(
                "appmail:render_message_body_html", kwargs={"email_id": obj.id}
            )
        return self.iframe(url)

    render_html.short_description = "HTML (rendered)"  # type: ignore
    render_html.allow_tags = True  # type: ignore



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['appmail', 'appmail.migrations']

package_data = \
{'': ['*'],
 'appmail': ['templates/*',
             'templates/admin/appmail/emailtemplate/*',
             'templates/admin/appmail/loggedmessage/*',
             'templates/appmail/*',
             'tests/fixtures/*']}

install_requires = \
['django>=3.0,<4.0']

setup_kwargs = {
    'name': 'django-appmail',
    'version': '2.0b4',
    'description': 'Django app for managing localised email templates.',
    'long_description': '# Django-AppMail\n\nDjango app for managing transactional email templates.\n\n## Compatibility\n\nThis project now requires Django 3.0+ and Python 3.7+. If you require a previous\nversion you will have to refer to the relevant branch or tag.\n\n## Background\n\nThis project arose out of a project to integrate a large transactional Django\napplication with Mandrill, and the lessons learned. It also owes a minor h/t\nto this project from 2011 (https://github.com/hugorodgerbrown/AppMail).\n\nThe core requirement is to provide an easy way to add / edit email templates\nto a Django project, in such a way that it doesn\'t require a developer to make\nchanges. The easiest way to use templated emails in Django is to rely on the\nin-built template structure, but that means that the templates are held in\nfiles, under version control, which makes it very hard for non-developers to\nedit.\n\nThis is **not** a WYSIWYG HTML editor, and it doesn\'t do anything clever. It\ndoesn\'t handle the sending of the emails - it simply provides a convenient\nmechanism for storing and rendering email content.\n\n```python\nfrom appmail.models import EmailTemplate, AppmailMessage\n\ndef send_order_confirmation(order_id):\n    order = Orders.objects.get(id=order_id)\n    template = EmailTemplate.objects.current(\'order_confirmation\')\n    context = { "order": order }\n    message = AppmailMessage(\n        template=template,\n        context=context,\n        to=[order.recipient.email]\n    )\n    message.send()\n```\n\nThe core requirements are:\n\n1. List / preview existing templates\n2. Edit subject line, plain text and HTML content\n3. Use standard Django template syntax\n4. Support base templates\n5. Template versioning\n6. Language support\n7. Send test emails\n\n**Template properties**\n\nIndividual templates are stored as model objects in the database. The\nstandard Django admin site is used to view / filter templates. The templates\nare ordered by name, language and version. This combination is unique. The\nlanguage and version properties have sensible defaults (\n`version=settings.LANGUAGE_CODE` and `version=0`) so don\'t need to set if you\ndon\'t require it. There is no inheritance or relationship between different\nlanguages and versions - they are stored as independent objects.\n\n```python\n# get the default order_summary email (language = settings.LANGUAGE_CODE)\ntemplate = EmailTemplate.objects.current(\'order_summary\')\n# get the french version\ntemplate = EmailTemplate.objects.current(\'order_summary\', language=\'fr\')\n# get a specific version\ntemplate = EmailTemplate.objects.version(\'order_summary\', 1)\n```\n\n**Template syntax**\n\nThe templates themselves use standard Django template syntax, including\nthe use of tags, filters. There is nothing special about them, however there\nis one caveat - template inheritance.\n\n**Template inheritance**\n\nAlthough the template content is not stored on disk, without re-engineering\nthe template rendering methods any parent templates must be. This is annoying,\nbut there is a valid assumption behind it - if you are changing your base\ntemplates you are probably involving designers and developers already, so\nhaving to rely on a developer to make the changes is acceptable.\n\n**Sending test emails**\n\nYou can send test emails to an email address through the admin list view.\n\n<img src="screenshots/appmail-test-email-action.png" alt="EmailTemplate admin change form" />\n\nThe custom admin action \'Send test emails\' will redirect to an intermediate\npage where you can enter the recipient email address and send the email:\n\n<img src="screenshots/appmail-test-email-send.png"/>\n\nThere is also a linkon individual template admin pages (top-right, next to the history link):\n\n<img src="screenshots/appmail-template-change-form.png" alt="EmailTemplate admin change form" />\n\n## Tests\n\nThere is a test suite for the app, which is best run through ``tox``.\n\n## License\n\nMIT\n\n## Contributing\n\nUsual rules apply:\n\n1. Fork to your own account\n2. Fix the issue / add the feature\n3. Submit PR\n\nPlease take care to follow the coding style - and PEP8.\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
